% File: ComputeW0.m
% Date: Jun 17, 2014
% Authors: anca.ciurte@cs.utcluj.ro (Anca Ciurte) and olivier.cuisenaire@gmail.com
% (Olivier Cuisenaire) and xavier.bresson@unil.ch (Xavier Bresson)
% Version: 2.0 beta
% License: GPLv3 - 29 June 2007
%
% Please cite the following paper:
% [1] A. Ciurte, X. Bresson, O. Cuisenaire, N. Houhou, S. Nedevschi, J-Ph. Thiran,
% M. Bach Cuadra, "Semi-Supervised Segmentation of Ultrasound Images based
% on Patch Representation and Continuous Min Cut", PlosOne, 9(7), p. e100972, 2014.
% Short Summary:
% --------------------------------------------------------------------------------------
%
% Copyright (c) 2014, anca.ciurte@cs.utcluj.ro (Anca Ciurte), olivier.cuisenaire@gmail.com
% (Olivier Cuisenaire) and xavier.bresson@unil.ch (Xavier Bresson)
% with Radiology Department, Medical Image Analysis Laboratory (MIAL), CIBM (UNIL
% and CHUV), Signal Processing Laboratory (LTS5 - EPFL),
% and Computer Science Department, Technical University of Cluj-Napoca
% (UTCN). 
% All rights reserved.
%
% This file is part of Fast Patch-based Continuos Min-Cut (fP-CMC)
% segmentation software
%
% fP-CMC is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% fP-CMC is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with fP-CMC. If not, see <http://www.gnu.org/licenses/>.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
% "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
% LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
% A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
% OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
% LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
% DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
% THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
% (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


function [GW]=ComputeW0(method,input,t,f,h,hv,hm,threshold,Options)
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                               
 %ComputeW0 builds a weighted graph

 %INPUT
 %  method: 1 (Gaussian additive noise), 2 (speckle noise)
 %  input: image on which the graph is constructed
 %  t: radio of search window
 %  f: size of the patch. If f=1 just the center pixel is used
 %  h: degree of filtering for distance between 2 patches
 %  hv: degree of filtering for local mean between 2 patches
 %  hm: degree of filtering for local variance between 2 patches
 %  threshold: Number of "best" neighbors
 %  Option:
 %  'distance': euclidean distance between patches (gray-level)
 %  'variance':  local variance on a patch
 %  'mean'    :  local mean on a patch  

  %Example:

  %[GW]=ComputeW0(Im0,11,3,0.2,0.002,0.001,struct('variance',1,'mean',2,'distance',3 ));%
  %OUTPUT: Structure of graph
  
  %GW.weight 
  %GW.position
  %size(GW.weight)=size(input)

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 

 input(input==0)=1;
 
 % Size of the image
 Opts.Variance= false;
 Opts.Mean= false;
 Opts.PatchDistance=false;
 [m n]=size(input);
 s1 = struct('variance',1);
 s2 = struct('mean',2);
 G = fspecial('gaussian', f, 1);
 if nargin>6
     fn = fieldnames(Options);
     
  
  for j=1:length(fn)
    name = fn{j};
    value = getfield(Options, name);
    if strcmpi(name, 'variance')  
        Opts.Variance = true;
    end
      if strcmpi(name, 'mean')  
        Opts.Mean = true;
      end

      if strcmpi(name, 'distance')  
        Opts.PatchDistance = true;
      end
 end  

 end

 % Memory for the output
 p = (t-1)/2;
 d = (f-1)/2;
 pad = d;

 % Replicate the boundaries of the input image
 input2 = padarray(input,[pad pad],'symmetric');
 mask2 = padarray(input,[pad pad],'symmetric');
 [m2 n2]=size(input2);
 maxIm = max(input(:));
 X=0:maxIm;
 std_pwm=1;
 Ng=ceil(6*std_pwm); 
 if (mod(Ng,2)==0); Ng=Ng+1; end;  Nc=round(Ng/2);
 z=1:Ng; G1D = exp(-(z-Nc).^2/(2*std_pwm^2))/ sqrt(2*pi*std_pwm^2);

 for i=1:m 
     for j=1:n
        i1 = i + d;
        j1 = j + d;
        W1= input2(i1-d:i1+d , j1-d:j1+d);
        if Opts.Mean
            meanW(i,j)=mean(W1(:));
        end
        if Opts.Variance 
            varW(i,j) = var(W1(:));
        end
        if Opts.PatchDistance
            W1=sort(W1(:));
            PatchW(i,j,:)=W1;

        end
     end
 end

 GW.weight = zeros(m,n,threshold); 
 GW.position = zeros(m,n,threshold); 
 
 for i=1:m 
     for j=1:n

        if Opts.Mean
            meanW1=meanW(i,j);
        end
        
        if Opts.Variance
            varW1 = varW(i,j);
        end

        if Opts.PatchDistance 
            W1 = (PatchW(i,j,:));
        end

        indice=[];
        weight=[];

        for r=i-p:1:i+p
         for s=j-p:1:j+p
            dist=0;
            rr = r;
            ss = s;
            if (r<1) rr=abs(r)+1;end
            if (s<1) ss=abs(s)+1;end
            if (r>m) 
                 rr=m-(r-m-1);
            end
            if (s>n) 
                ss=n-(s-n-1);
            end

            W2 = (PatchW(rr,ss,:));
            card = size(W1,1)*size(W1,1);
            switch method
                case 1
                    dist =  sum(sum((W1-W2).*(W1-W2)))/(card*h);
                case 2
                    dist = sum(sum((W1-W2).*(W1-W2)./(2*card*h*W2)));
            end
            w=exp(-dist); 

            weight=[weight,w];
            indice = [indice;[rr,ss]];         
         end 
        end
        
        [val ind]=sort(weight,'descend');
        GW.weight(i,j,:)=val(1:threshold);
        k=[indice(ind(1:threshold),1),indice(ind(1:threshold),2)];  
        GW.position(i,j,:)=sub2ind([m n],k(:,1),k(:,2));
 %        end
     end
 end

 

%%

function [dy]=Forward(u,d);
[Ny,Nx] = size(u);
dy = zeros(d,d);
dy(1:Ny-1)= u(2:Ny) - u(1:Ny-1) ;

 
function [dx]=ForwardX(u,f);
[Ny,Nx] = size(u);
dx = zeros(f,f);
dx(1:Ny-1,1:Nx-1)=( u(1:Ny-1,2:Nx) - u(1:Ny-1,1:Nx-1) );


function [dy]=ForwardY(u,f);
[Ny,Nx] = size(u);
dy = zeros(f,f);
dy(1:Ny-1,1:Nx-1)=( u(2:Ny,1:Nx-1) - u(1:Ny-1,1:Nx-1) );