% File: main.m
% Date: Jun 17, 2014
% Authors: xavier.bresson@gmail.com (Xavier Bresson), anca.ciurte@cs.utcluj.ro (Anca Ciurte) and
% olivier.cuisenaire@gmail.com (Olivier Cuisenaire)
% Version: 2.0 beta
% License: GPLv3 - 29 June 2007
%
% Please cite the following paper:
% [1] A. Ciurte, X. Bresson, O. Cuisenaire, N. Houhou, S. Nedevschi, J-Ph. Thiran,
% M. Bach Cuadra, "Semi-Supervised Segmentation of Ultrasound Images based
% on Patch Representation and Continuous Min Cut", PlosOne, 9(7), p. e100972, 2014.
% Short Summary:
% --------------------------------------------------------------------------------------
%
% Copyright (c) 2014, anca.ciurte@cs.utcluj.ro (Anca Ciurte), olivier.cuisenaire@gmail.com
% (Olivier Cuisenaire) and xavier.bresson@unil.ch (Xavier Bresson)
% with Radiology Department, Medical Image Analysis Laboratory (MIAL), CIBM (UNIL
% and CHUV), Signal Processing Laboratory (LTS5 - EPFL),
% and Computer Science Department, Technical University of Cluj-Napoca
% (UTCN). 
% All rights reserved.
%
% This file is part of Fast Patch-based Continuos Min-Cut (fP-CMC)
% segmentation software
%
% fP-CMC is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% fP-CMC is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with fP-CMC. If not, see <http://www.gnu.org/licenses/>.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
% "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
% LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
% A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
% OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
% LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
% DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
% THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
% (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PARAMETERS TO BE SET:                                             %
%                                                                   %
% t: size of the searching window
% f: size of the patch.                                             %
% sigma2: scaling parameter
% beta: regularization weigth                                       %
%                                                                   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear, close all
fprintf('\nBegin processings...\n');
fprintf('\tChoose the image file that you want to segment.\n');
[fname , dirname ] = uigetfile('*.png; *.pgm; *.tif; *.bmp','Input Mask');
Im0 = imread(fullfile(dirname,fname));
Im0 = imresize(Im0, [200, 200*size(Im0,2)/size(Im0,1)]);

dim =  size(size(Im0));
if (dim(2)==3) 
   Im0 = rgb2gray(Im0); 
end
Im0 = double(Im0)./max(double(Im0(:)));

filename = fname(1:length(fname)-4);
filename_u = sprintf('%su_%s', dirname, filename); 
filename_w = sprintf('%sw_%s.mat', dirname, filename); 

beta = 1e-2; 
t1 = 0;

% /***********   Computes the graph of patches w   **********/
if (exist(filename_w)==0) %erase the .mat file if you want to recompute with other parameters
    t = 5;
    f = 3;
    sigma2 = 0.007; 
    fprintf('\tComputing the graph of patches w...\n');
    tic
    run(Im0, dirname, filename_w, t, f, sigma2, beta);
    t1 = toc; 
    s = sprintf('\tGraph w succesfully created! (%4.2f seconds)\n', t1); 
    fprintf(s);
else 
    fprintf('\tGraph of patches w pre-computed!\n');
end

% /***********       Segmentation with P-CMC     **********/

% /***********    labels definition    **********/
in ='';
while 1
    fprintf('\tSelect the initial labels!\n');
    fprintf('\tChoose the type of label definition:\n');
    fprintf('\t\t1 - polyline for object, ellipse for background\n');
    fprintf('\t\t2 - ellipses for both labels\n');
    fprintf('\t\t3 - polylines for both labels:\t');
    in=input('','s');
    if (in=='1' || in=='2' || in=='3') break; end
end

switch in
    case '1'
        fprintf('\t\tObject label: draw the label as a polygonal line inside the object followed by a double click.\n');
        figure(10), clf, imshow(Im0,[]); hold on,
        truesize(10,[round(2*size(Im0,1)) round(2*size(Im0,2))]);
        iptsetpref('Imshowborder','tight');
        label_obj = roipoly();
        fprintf('\t\tBackground label: draw the label as an ellipse outside the object followed by a key pressed.\n');
        figure(10), h_bk = imellipse(); pause();
        label_bk = 1-createMask(h_bk);
    case '2'
        fprintf('\t\tObject label: draw the label as as an ellipse outside the object followed by a key pressed.\n');
        figure(10), clf, imshow(Im0,[]); hold on,
        truesize(10,[round(2*size(Im0,1)) round(2*size(Im0,2))]);
        iptsetpref('Imshowborder','tight');
        h_obj = imellipse(); pause();
        label_obj = createMask(h_obj);
        fprintf('\t\tBackground label: draw the label as an ellipse outside the object followed by a key pressed.\n');
        figure(10), h_bk = imellipse(); pause();
        label_bk = 1-createMask(h_bk);
    case '3'
        fprintf('\t\tObject label: draw the label as a polygonal line inside the object followed by a double click.\n');
        figure(10), clf, imshow(Im0,[]); hold on,
        truesize(10,[round(2*size(Im0,1)) round(2*size(Im0,2))]);
        iptsetpref('Imshowborder','tight');
        label_obj = roipoly();      
        fprintf('\t\tBackground label: draw the label as a polygonal line inside the object followed by a double click.\n');
        figure(10), label_bk = 1 - roipoly(); 
end

labels = ones(size(Im0)).*0.5;
labels(label_obj==1)=1; labels(label_bk==1)=0;


% /***********   Segmentation   **********/
fprintf('\tComputing the graph flow...\n')
tic
test_semi_superv_seg(labels, filename_w, filename_u, beta); %,
t2 = toc; 
s = sprintf('The segmentation result is marked in Fig. 100 in magenta! (%4.2f seconds)\n', t2); 
fprintf(s);

s = sprintf('End processings! (Total time = %4.2f seconds)\n\n', (t1+t2));
fprintf(s);

