% File: test_semi_superv_seg.m
% Date: Jun 17, 2014
% Authors: anca.ciurte@cs.utcluj.ro (Anca Ciurte) and olivier.cuisenaire@gmail.com
% (Olivier Cuisenaire) and xavier.bresson@unil.ch (Xavier Bresson)
% Version: 2.0 beta
% License: GPLv3 - 29 June 2007
%
% Please cite the following paper:
% [1] A. Ciurte, X. Bresson, O. Cuisenaire, N. Houhou, S. Nedevschi, J-Ph. Thiran,
% M. Bach Cuadra, "Semi-Supervised Segmentation of Ultrasound Images based
% on Patch Representation and Continuous Min Cut", PlosOne, 9(7), p. e100972, 2014.
% Short Summary:
% --------------------------------------------------------------------------------------
%
% Copyright (c) 2014, anca.ciurte@cs.utcluj.ro (Anca Ciurte), olivier.cuisenaire@gmail.com
% (Olivier Cuisenaire) and xavier.bresson@unil.ch (Xavier Bresson)
% with Radiology Department, Medical Image Analysis Laboratory (MIAL), CIBM (UNIL
% and CHUV), Signal Processing Laboratory (LTS5 - EPFL),
% and Computer Science Department, Technical University of Cluj-Napoca
% (UTCN). 
% All rights reserved.
%
% This file is part of Fast Patch-based Continuos Min-Cut (fP-CMC)
% segmentation software
%
% fP-CMC is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% fP-CMC is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with fP-CMC. If not, see <http://www.gnu.org/licenses/>.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
% "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
% LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
% A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
% OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
% LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
% DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
% THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
% (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


function test_wGrB(labels, filename_w, filename_u, beta)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 % test_wGrB.m perform the graph segmentation with fP-CMC
 % and saves the results in filename_u.mat file

 % INPUT
 % labels: an array of the same size as Im0 containing the initial labels
 %         (1 - for object and 0 - for background)
 % filename_w: the name of the file where the weihted graph is saved
 % filename_u: the name of the file where the segmentation results will be
 %             saved
 
 % Example:

 % test_semi_superv_seg(labels, filename_w, filename_u, beta);
 % OUTPUT: none (the segmentation result is saved in filename_w.mat)
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load the input image
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

YES = 0;     NO = 1;

cpt_fig = 0;

load(filename_w, 'Im0', 'SparseW', 'GW', 'param');

[Ny,Nx] = size(Im0);
n = Nx*Ny;
f2=1.5; %affects figure size


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% collect patches
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

w  = SparseW; clear SparseW;
wv = dn(w,'ave'); 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% compute gradient operator
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

wt = triu(wv,1); % upper triangular part of w
[r c v] = find(wt);
n = length(r);
Dr = [1:n 1:n];
Dc(1:n) = r;
Dc(n+1:2*n) = c;
Dv(1:n) = -2*sqrt(v);
Dv(n+1:2*n) = 2*sqrt(v);
D = sparse(Dr,Dc,Dv);
Dt = D';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Matrix for derivative operators as finite differences
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
n = Nx*Ny;

% x-forward derivative operator
ep = ones(n,1);
em = zeros(n,1); em(1:n-Ny)=-1; % boundary condition Dxf=0 at x=Nx
Dfx = spdiags([em ep],[0 Ny],n,n);

% y-forward derivative operator
I = 1:n; [I J] = find(mod(I,Ny)==0);   % extract the last row, i.e. y=Ny
ep = ones(n,1); ep(J+1)=0; ep=ep(1:n); % boundary condition Dyf=0 at y=Ny
em = -ones(n,1); em(J)=0;
Dfy = spdiags([em ep],[0 1],n,n);

% x-backward divergence operator
% boundary condition Dxb=u_x at x=1, Dxb=u_{x-1} at x=Nx,
ep = zeros(n,1); ep(1:n-Ny)=1;
em = -ones(n,1);
Dbx = spdiags([ep em],[0 -Ny],n,n);

% y-forward derivative operator
% boundary condition Dyb=u_y at y=1, Dyb=u_{y-1} at y=Ny,
I = 1:n; [I J] = find(mod(I,Ny)==0);
ep = ones(n,1); ep(J)=0;
em = -ones(n,1); em(J)=0; em=em(1:n);
Dby = spdiags([ep em],[0 -1],n,n);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

nsi = 100;
r1 = 0.5; r2 = 0.5;
r1inv = 1/r1; r2inv = 1/r2;

u = zeros(size(Im0)); u(labels==1) = 1;
v = zeros(size(Im0)); v(labels==1) = 1;   
qx = zeros(size(Im0));
qy = zeros(size(Im0));

l1 = zeros(size(Im0));
l2x = zeros(size(Im0));
l2y = zeros(size(Im0));

%r13c = reshape(r13,Ny*Nx,1);
md=@(x,type) Dt*(D*x)+r1*x; % anonymous function for multiplication by q1D'D+q2I

[X,Y] = meshgrid(1:Nx,1:Ny); 
auxFFT = cos(2*pi/Nx*X)+cos(2*pi/Ny*Y)-2;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Iterative algorithm
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

u = rand(size(Im0));
u ( labels == 0 ) = 0;  
u ( labels == 1 ) = 1;  

u_old = zeros(size(u));

nIters = 10000;
vareps = 1e-6;
i=0;

while (sum((abs(u(:)-u_old(:)).^2))/(size(Im0,1)*size(Im0,2)) > vareps) & (i<nIters)    
   
    i = i+1;
    u = reshape(u,Ny,Nx); 
    u_old=u;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%
    % v^n+1
    %%%%%%%%%%%%%%%%%%%%%%%%%
 

    b = r1* (u+ r1inv* l1);
    b = b -r2* ( Bx(qx+r2inv*l2x,Dbx)+By(qy+r2inv*l2y,Dby) );
    v = real(ifft2( fft2( b )./( -2*r2*auxFFT + r1 )));
  

    %%%%%%%%%%%%%%%%%%%%%%%%%
    % q^n+1
    %%%%%%%%%%%%%%%%%%%%%%%%%
    Fxv = Fx(v,Dfx);
    Fyv = Fy(v,Dfy);
    ex = Fxv - r2inv* l2x;
    ey = Fyv - r2inv* l2y;
    [qx,qy] = shrink(ex,ey,beta*r2inv); 
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%
    % v^n+1
    %%%%%%%%%%%%%%%%%%%%%%%%%

    vv = r1*v-l1; vv = reshape(vv,Ny*Nx,1);
    u = reshape(u,Ny*Nx,1);
    [u flag]=pcg(md,vv,1e-10,nsi,[],[],u); % step towards v^n+1 using conjugate gradient
    u = reshape(u,Ny,Nx);
    u(labels==0) = 0;
    u(labels==1) = 1; 
    
    u(u<0)=0; u(u>1)=1; 
   
    
    %%%%%%%%%%%%%%%%%%%%%
    % update lambda
    %%%%%%%%%%%%%%%%%%%%%
    
    l1 = l1 + r1*(u-v);
    l2x = l2x + r2*(qx-Fx(v,Dfx));
    l2y = l2y + r2*(qy-Fy(v,Dfy));

    
    if rem(i,1)==0      
        [r,c]=find(v>0.01); s=0; for j=1:size(r,1); s=s+v(r(j),c(j)); end; mean=s/size(r,1);
        cpt_fig = 100;
        figure(cpt_fig); clf;
        imagesc(Im0); colormap(gray); title(i); hold on;
        [c,h] = contour(u,[0.5 0.5],'m-'); set(h,'LineWidth',1);
        truesize(cpt_fig,[round(f2*Ny) round(f2*Nx)]);
        pause(0.1)

    end
    pause(0.01)
    
end

%save(filename_u,'u', 'labels', 'Im0',  'i');



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% forward/backward/upwind differencing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function Fxu = Fx(u,Dfx)
% Forward derivative operator on x

[Ny,Nx] = size(u);

% reshape u
n = Nx*Ny;
u = reshape(u,Nx*Ny,1);

% Dfx
Fxu = Dfx*u;
Fxu = reshape(Fxu,Ny,Nx);

% reshape u
u = reshape(u,Ny,Nx);



function Fyu = Fy(u,Dfy)
% Forward derivative operator on y

[Ny,Nx] = size(u);

% reshape u
n = Nx*Ny;
u = reshape(u,Nx*Ny,1);

% Dfy
Fyu = Dfy*u;
Fyu = reshape(Fyu,Ny,Nx);
 
% reshape u
u = reshape(u,Ny,Nx);



function Bxu = Bx(u,Dbx)
% Backward derivative operator on x

[Ny,Nx] = size(u);

% reshape u
n = Nx*Ny;
u = reshape(u,Nx*Ny,1);

% Dbx
Bxu = Dbx*u;
Bxu = reshape(Bxu,Ny,Nx);

% reshape u
u = reshape(u,Ny,Nx);

 

function Byu = By(u,Dby)
% Backward derivative operator on y

[Ny,Nx] = size(u);

% reshape u
n = Nx*Ny;
u = reshape(u,Nx*Ny,1);

% Byu
Byu = Dby*u;
Byu = reshape(Byu,Ny,Nx);

% reshape u
u = reshape(u,Ny,Nx);




function [xs,ys] = shrink(x,y,lambda)
% shrinkage, ignore the condition max{0,*}

s = sqrt(x.*conj(x)+y.*conj(y));
ss = s-lambda;
ss = ss.*(ss>0);

s = s+(s<lambda);
ss = ss./s;

xs = ss.*x;
ys = ss.*y;

return;




